/**
 * Copyright (C) 2018-2019 thirty bees
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@thirtybees.com so we can send you a copy immediately.
 *
 * @author    thirty bees <modules@thirtybees.com>
 * @copyright 2018-2019 thirty bees
 * @license   Academic Free License (AFL 3.0)
 */

/*
 * Upgrade panel.
 */
var gitUpdaterParameters;

$(document).ready(function () {
  gitUpdaterParameters = JSON.parse($('input[name=GIT_UPDATER_PARAMETERS]').val());

  channelChange();
  $('#GIT_UPDATER_CHANNEL').on('change', channelChange);

  $('#GIT_UPDATER_VERSION').on('change', versionChange);

  addBootstrapCollapser('GIT_UPDATER_PROCESSING', false);

  if (document.getElementById('configuration_fieldset_comparepanel')) {
    processCompare();
  }
});

function channelChange() {
  let channel = $('#GIT_UPDATER_CHANNEL option:selected').val();
  let versionSelect = $('#GIT_UPDATER_VERSION');

  if ( ! channel || ! versionSelect.length) {
    return;
  }

  versionSelect.empty();
  $.ajax({
    url: gitUpdaterParameters.apiUrl,
    type: 'POST',
    data: {'list': channel},
    dataType: 'json',
    success: function(data, status, xhr) {
      data.forEach(function(version) {
          versionSelect.append('<option>'+version+'</option>');
          if (version === gitUpdaterParameters.selectedVersion) {
            versionSelect.val(gitUpdaterParameters.selectedVersion);
          }
      });
      $('#conf_id_GIT_UPDATER_VERSION')
        .find('.help-block')
        .parent()
        .slideUp(200);
    },
    error: function(xhr, status, error) {
      let helpText = $('#conf_id_GIT_UPDATER_VERSION').find('.help-block');
      helpText.html(gitUpdaterParameters.errorRetrieval);
      helpText.css('color', 'red');
      console.log('Request to '+gitUpdaterParameters.apiUrl
                  +' failed with status \''+xhr.state()+'\'.');
    },
  });
}

function versionChange() {
  if ($(this).val() === gitUpdaterParameters.selectedVersion) {
    $('#configuration_fieldset_comparepanel').slideDown(1000);
  } else {
    $('#configuration_fieldset_comparepanel').slideUp(1000);
  }
}

function processCompare() {
  $.ajax({
    url: document.URL+'&action=processCompare&ajax=1',
    type: 'POST',
    data: {'compareVersion': gitUpdaterParameters.selectedVersion},
    dataType: 'json',
    success: function(data, status, xhr) {
      logField = $('textarea[name=GIT_UPDATER_PROCESSING]')[0];
      infoList = data['informations'];
      infoListLength = infoList.length;

      for (i = 0; i < infoListLength; i++) {
        logField.value += "\n";
        if (data['error'] && i === infoListLength - 1) {
          logField.value += "ERROR: ";
          $('#conf_id_GIT_UPDATER_PROCESSING')
            .children('label')
            .html(gitUpdaterParameters.errorProcessing)
            .css('color', 'red');
        }
        logField.value += data['informations'][i];
      }

      logField.scrollTop = logField.scrollHeight;

      if (data['changeset']) {
        changesets = data['changeset'];
        if (changesets['change']) {
          appendChangeset(changesets['change'], 'GIT_UPDATER_UPDATE');
          addBootstrapCollapser('GIT_UPDATER_UPDATE', true);
        }
        if (changesets['add']) {
          appendChangeset(changesets['add'], 'GIT_UPDATER_ADD');
          addBootstrapCollapser('GIT_UPDATER_ADD', true);
        }
        if (changesets['remove']) {
          appendChangeset(changesets['remove'], 'GIT_UPDATER_REMOVE');
          addBootstrapCollapser('GIT_UPDATER_REMOVE', true);
        }
        if (changesets['obsolete']) {
          appendChangeset(changesets['obsolete'], 'GIT_UPDATER_REMOVE_OBSOLETE');
          addBootstrapCollapser('GIT_UPDATER_REMOVE_OBSOLETE', true);
        }
      }

      if (data['done'] === false) {
        processCompare();
      } else {
        $('#collapsible_GIT_UPDATER_PROCESSING').collapse('hide');
        addCompletedText('GIT_UPDATER_PROCESSING',
                         gitUpdaterParameters.completedLog);
      }
    },
    error: function(xhr, status, error) {
      $('#configuration_fieldset_comparepanel')
        .children('.form-wrapper')
        .html(gitUpdaterParameters.errorRetrieval)
        .css('color', 'red');
      console.log('Request to '+document.URL+'&action=processCompare&ajax=1'
                  +' failed with status \''+xhr.state()+'\'.');
    }
  });
}

function appendChangeset(changeset, field) {
  let node = $('#conf_id_'+field);

  let html = '<table class="table"><tbody>';

  let count = 0;
  for (line in changeset) {
    count++;
    html += '<tr>'
    if (field !== 'GIT_UPDATER_REMOVE_OBSOLETE') {
      if (changeset[line]) {
        html += '<td>M</td>';
      } else {
        html += '<td>&nbsp;</td>';
      }
    } else {
      html += '<td><input type="checkbox"></td>'
    }
    html += '<td>'+line+'</td>';
    html += '</tr>'
  }
  if ( ! count) {
    html += '<tr><td>-- none --</td></tr>';
  }

  html += '</tbody></table>';

  node.append(html);

  addCompletedText(field, gitUpdaterParameters.completedList, count);
}

function addCompletedText(field, text, number) {
  let element = $('#conf_id_'+field).children('label');
  if (element.children('a').length) {
    element = element.children('a');
  }

  if (number !== undefined) {
    text = text.replace('%d', number);
  }

  let string = element[0].innerHTML.trim();
  let colon = string.slice(-1);
  if (colon === ':') {
    string = string.slice(0, -1);
  }
  string += ' ('+text+')';
  if (colon === ':') {
    string += ':';
  }

  element[0].innerHTML = string;
}

function addBootstrapCollapser(field, initiallyCollapsed) {
  let trigger = $('#conf_id_'+field).children('label');
  let collapsible = $('#conf_id_'+field).children().last();
  let collapsibleName = 'collapsible_'+field;

  let iconClass = 'icon-collapse-alt';
  if (initiallyCollapsed) {
    iconClass = 'icon-expand-alt';
  }
  trigger[0].innerHTML = '<a data-toggle="collapse"'
                         +'  data-target="#'+collapsibleName+'"'
                         +'  style="color: inherit; text-decoration: inherit;">'
                         +'<i class="'+iconClass+'"></i>'
                         +' '
                         +trigger[0].innerHTML.trim()
                         +'</a>';

  collapsible[0].id = collapsibleName;
  collapsible[0].className += ' collapse';
  if ( ! initiallyCollapsed) {
    collapsible[0].className += ' in';
  }

  collapsible.on("hide.bs.collapse", function(){
    $(this).siblings('label').find('i')[0].className = 'icon-expand-alt';
  });
  collapsible.on("show.bs.collapse", function(){
    $(this).siblings('label').find('i')[0].className = 'icon-collapse-alt';
  });
}
